/*
 * 系统设置权限管理路由表
 * */
import layout from '@/components/layout/layout'
export default [
  // 系统设置
  {
    path: '/systemManagement',
    component: layout,
    children: [
      // 代理登陆
      {
        path: 'proxyAdminLogin',
        component: () =>
          import('@/views/common/login/proxyAdminLogin.vue')
      },
      // 管理
      {
        path: 'templateLibrary',
        meta: { path: 'templateLibrary' },
        component: () =>
          import('@/views/common/systemManagement/templateLibrary/templateLibrary-list')
      },
      // 范本库管理
      {
        path: 'templateLibrary',
        meta: { path: 'templateLibrary' },
        component: () =>
          import('@/views/common/systemManagement/templateLibrary/templateLibrary-list')
      },
      // 系统配置管理
      {
        path: 'systemConfig-list',
        meta: { path: 'systemConfig-list' },
        component: () =>
          import('@/views/common/systemManagement/systemConfig/systemConfig-list')
      },
      // 系统文档管理
      {
        path: 'systemZipDoc-list',
        meta: { path: 'systemZipDoc-list' },
        component: () =>
          import('@/views/common/systemManagement/systemConfig/systemZipDoc-list')
      },
      // 配置管理
      {
        path: 'platformConfig-list',
        meta: { path: 'platformConfig-list' },
        component: () =>
          import('@/views/common/systemManagement/platformConfig/platformConfig-list')
      },
      // 用户管理
      {
        path: 'userManagement',
        meta: { path: 'userManagement' },
        component: () =>
          import('@/views/common/systemManagement/userManage/userManage-list')
      },
      // 菜单管理
      {
        path: 'menuManagement',
        meta: { path: 'menuManagement' },
        component: () =>
          import('@/views/common/systemManagement/menu/menu-list')
      },
      // 角色管理
      {
        path: 'roleManagement',
        meta: { path: 'roleManagement' },
        component: () =>
          import('@/views/common/systemManagement/role/role-list')
      },
      // 数据字典
      {
        path: 'dictionary',
        meta: { path: 'dictionary' },
        component: () =>
          import('@/views/common/systemManagement/dictionary/dictionary-list')
      },
      // 国际化语言
      {
        path: 'internationalLanguage',
        meta: { path: 'internationalLanguage' },
        component: () => import('@/views/common/systemManagement/language/language-list')
      },
      // 采购单位新注册
      {
        path: 'orgRegisterInfo',
        meta: { path: 'orgRegisterInfo' },
        component: () => import('@/views/common/systemManagement/organization/orgRegisterInfo/orgRegisterInfo-add.vue')
      },
      // 招标代理机构新注册
      {
        path: 'signOrgRegisterInfo',
        meta: { path: 'signOrgRegisterInfo' },
        component: () => import('@/views/common/systemManagement/organization/orgRegisterInfo/orgRegisterInfo-add.vue')
      },
      // 机构审核列表
      {
        path: 'organizationAudit',
        meta: { path: 'organizationAudit' },
        component: () => import('@/views/common/systemManagement/organization/organizationAudit/organizationAudit-list.vue')
      },
      // 机构审核详情
      {
        path: 'organizationAuditDetail',
        name: 'OrganizationAuditDetail',
        meta: { path: 'organizationAudit' },
        component: () => import('@/views/common/systemManagement/organization/organizationAudit/organizationAudit-detail.vue')
      },
      // 组织机构列表
      {
        path: 'organization',
        meta: { path: 'organization' },
        component: () => import('@/views/common/systemManagement/organization/organizationList/organization-list.vue')
      },
      // 组织机构新增部门
      {
        path: 'addSectors',
        meta: { path: 'organization' },
        component: () => import('@/views/common/systemManagement/organization/addSectors/sectors-add')
      },
      // 查看机构
      {
        path: 'lookMachine',
        meta: { path: 'organization' },
        component: () => import('@/views/common/systemManagement/organization/lookMachineMsg/content'),
        children: [
          // 查看公司基本信息
          {
            path: 'companyInfo',
            component: () => import('@/views/common/systemManagement/organization/lookMachineMsg/companyInfo-detail')
          },
          // 联系人信息
          {
            path: 'contactInfo',
            component: () => import('@/views/common/systemManagement/organization/lookMachineMsg/contactInfo-detail')
          },
          // 财务信息
          {
            path: 'financeInfo',
            component: () => import('@/views/common/systemManagement/organization/lookMachineMsg/financeInfo-detail')
          }
        ]
      },
      // 组织机构新增机构
      {
        path: 'addOrganization',
        meta: { path: 'organization' },
        component: () => import('@/views/common/systemManagement/organization/addOrganization/content'),
        children: [
          {
            path: 'companyProfile',
            meta: { step: 1 },
            component: () => import('@/views/common/systemManagement/organization/addOrganization/company-add')
          },
          {
            path: 'contactInformation',
            meta: { step: 2 },
            component: () => import('@/views/common/systemManagement/organization/addOrganization/contactInformation-add')
          },
          {
            path: 'financialInformation',
            meta: { step: 3 },
            component: () => import('@/views/common/systemManagement/organization/addOrganization/financialInformation-add')
          }
        ]
      },
      // 按钮管理列表
      {
        path: 'buttonMgmt-list',
        meta: { path: 'buttonMgmt-list' },
        component: () =>
          import('@/views/common/systemManagement/buttonMgmt/buttonMgmt-list')
      }

    ]
  },
  // 业务设置
  {
    path: '/serviceConfig',
    component: layout,
    children: [
      // 参数管理
      {
        path: 'serviceConfiguration',
        meta: { path: 'serviceConfiguration' },
        component: () =>
          import('@/views/common/systemManagement/serviceConfig/serviceConfiguration')
      },
      // 规则变更列表
      {
        path: 'bidBulletinTenderStageRule',
        meta: { path: 'bidBulletinTenderStageRule' },
        component: () =>
          import('@/views/bidding/bulletin/tenderStageRuleExpand/tenderStageRuleExpand-list')
      },
      // 规则变更详情
      {
        path: 'bidBulletinTenderStageRuleUpdate',
        name: 'BidBulletinTenderStageRuleUpdate',
        meta: { path: 'bidBulletinTenderStageRule' },
        component: () =>
          import('@/views/bidding/bulletin/tenderStageRuleExpand/tenderStageRuleExpand-update')
      }
    ]
  }
]
